require 'ribbon/intercom/version'
require 'rack'

module Ribbon
  module Intercom
    require 'ribbon/intercom/railtie' if defined?(Rails)
    autoload(:Service,     'ribbon/intercom/service')
    autoload(:Errors,      'ribbon/intercom/errors')
    autoload(:Client,      'ribbon/intercom/client')
    autoload(:Package,     'ribbon/intercom/package')
    autoload(:Packageable, 'ribbon/intercom/packageable')
    autoload(:Packet,      'ribbon/intercom/packet')
    autoload(:Utils,       'ribbon/intercom/utils')

    module_function

    def load_tasks
      Dir[
        File.expand_path("../../tasks", __FILE__) + '/**.rake'
      ].each { |rake_file| load rake_file }
    end

    def method_missing(meth, *args, &block)
      client.send(meth, *args, &block)
    end

    def client
      @_client ||= Client.new
    end

    def mock_safe
      orig_client = client
      @_client = @_client.mock_safe
      yield
    ensure
      @_client = orig_client
    end
  end # Intercom
end # Ribbon

# Create a shortcut to the module
Intercom = Ribbon::Intercom