require 'ribbon/config'

module Ribbon::Intercom
  class Client
    autoload(:SDK, 'ribbon/intercom/client/sdk')
    autoload(:MockSDK, 'ribbon/intercom/client/mock_sdk')

    def config(&block)
      (@__config ||= Ribbon::Config.new).tap { |config|
        if block_given?
          config.define(&block)
        end
      }
    end

    def [](handle)
      _load_sdk(handle)
    end

    ##
    # Return a mock safe version of this client.
    def mock_safe
      dup.tap { |client|
        client.preload_sdks

        # Make SDK instances mock safe
        client.sdk_instances = Hash[
          client.sdk_instances.map { |service, sdk| [service, sdk.mock_safe] }
        ]
      }
    end

    def preload_sdks
      if config.service?
        config.service.each { |name, *args|
          _load_sdk(name)
        }
      end
    end

    protected

    attr_accessor :sdk_instances

    private

    def _load_sdk(handle)
      raise Errors::ServiceNotDefinedError unless config.service?

      (self.sdk_instances ||= Hash.new { |hash, key|
        # Get the last service if there are duplicates
        if (service_def = config.service.select { |r| r.first == key }.last)
          name, config = service_def
          hash[key] = SDK.new(config[:url], config[:token], config[:secret])
        else
          raise Errors::ServiceNotDefinedError
        end
      })[handle.to_sym]
    end
  end # Client
end # Ribbon::Intercom
