module Ribbon::Intercom
  module Client::SDK::Adapters
    class Adapter
      autoload(:Response, 'ribbon/intercom/client/sdk/adapters/adapter/response')

      def initialize(*args)
        connect(*args)
      end

      ##
      # Call the method on the service.
      #
      # @deprecated This method is only called by tests. Don't call it elsewhere.
      def call(method_name, *args)
        send_packet(
          method_queue: Packet::MethodQueue.new.enqueue(method_name, *args)
        )
      end

      ##
      # Send a packet to the service and returns the Packet returned by the service.
      def send_packet(packet)
        packet = Packet.new(packet) if packet.is_a?(Hash)

        send_packet!(packet.encode).tap { |response|
          raise TypeError, "send_packet! should return an Adapter::Response" unless response.is_a?(Response)
        }.packet
      end

      ##
      # Returns the headers defined for this Adapter. Optionally, you may also
      # define additional headers you'd like to add/override.
      def headers(new_headers={})
        (@headers ||= {}).merge!(new_headers)
      end

      def dup
        super.tap { |adapter|
          adapter.headers = @headers.dup if @headers
        }
      end

      ##
      # Connect to a service. The specific arguments depend on the Adapter
      # subclass.
      def connect(*args)
        raise NotImplementedError
      end

      ##
      # Returns whether or not the Adapter is connected to a service.
      def connected?
        raise NotImplementedError
      end

      ##
      # Actually send the packet to the service. Should return an Adapter::Response object.
      def send_packet!(packet)
        raise NotImplementedError
      end

      protected

      def headers=(h)
        @headers = h
      end

      private

      ##
      # Encode the arguments for transmission to the service.
      def _encode_args(*args)
        Base64.strict_encode64(Marshal.dump(Utils.sanitize(args)))
      end
    end # Adapter
  end # Client::SDK::Adapters
end # Ribbon::Intercom