require 'net/http'
require 'openssl'
require 'uri'

module Ribbon::Intercom
  module Client::SDK::Adapters
    class HttpAdapter::Connection
      attr_reader :url

      def initialize(url, token, secret)
        @url = url.is_a?(String) ? URI(url) : url

        @_client = Net::HTTP.new(@url.hostname, @url.port)
        @_client.use_ssl = @url.is_a?(URI::HTTPS)
        @_client.verify_mode = OpenSSL::SSL::VERIFY_PEER
        @_client.cert_store = OpenSSL::X509::Store.new.tap { |s| s.set_default_paths }

        @_headers = {
          'Authorization' => "Basic #{Base64.strict_encode64("#{token}:#{secret}")}"
        }
      end

      def put(params={})
        @_client.put(url.request_uri, params[:body], _prepare_headers(params[:headers]).merge(@_headers))
      end

      private

      def _prepare_headers(headers)
        Hash[(headers || {}).map { |k, v| [k.to_s, v.to_s] }]
      end
    end # HttpAdapter::Connection
  end # Client::SDK::Adapters
end # Ribbon::Intercom