require 'base64'

module Ribbon::Intercom
  module Client::SDK::Adapters
    class LocalAdapter < Adapter
      attr_reader :service
      attr_accessor :channel_token
      attr_accessor :channel_secret

      def connect(service, params={})
        case service
        when Service
          @service = service
        else
          raise ArgumentError, "Expected a service, got: #{service.inspect}"
        end

        @channel_token = params[:channel_token]
        @channel_secret = params[:channel_secret]
      end

      def connected?(*args)
        !!service
      end

      ##
      # Mimics an HTTP PUT
      def send_packet!(encoded_packet)
        response = service.call(
          _http_headers.merge(
            'HTTP_AUTHORIZATION'     => _http_auth,
            'REQUEST_METHOD'         => 'PUT',
            'rack.input'             => StringIO.new(encoded_packet)
          )
        )

        Adapter::Response.new(response[2], response[0], response[1])
      end

      private

      def _http_auth
        "Basic #{Base64.strict_encode64("#{channel_token}:#{channel_secret}")}"
      end

      def _http_headers
        Hash[headers.map { |name, value| [_header_to_http(name), value] }]
      end

      def _header_to_http(name)
        "HTTP_#{name.to_s.upcase.gsub('-', '_')}"
      end
    end # MockAdapter
  end # Client::SDK::Adapters
end # Ribbon::Intercom