module Ribbon::Intercom
  module Errors
    class Error < StandardError; end

    ##
    # Packet errors
    class PacketError < Error; end
    class InvalidEncodingError < PacketError; end

    #############
    # Http Errors
    class HttpError < Error; end

    ##
    # Request Errors

    # 400 Error
    class RequestError < HttpError; end
    class UnsafeArgumentError < RequestError; end
    class InvalidSubjectSignatureError < RequestError; end

    # 401 Error
    class AuthenticationError < RequestError; end

    # 403 Error
    class ForbiddenError < RequestError; end
    class InsufficientPermissionsError < ForbiddenError; end

    # 404 Not Found
    class NotFoundError < RequestError; end
    class InvalidMethodError < NotFoundError; end

    # 405 Method Not Allowed
    class MethodNotAllowedError < RequestError; end

    ##
    # Server Errors

    # 500 Error
    class ServerError < HttpError; end
    class UnsafeResponseError < ServerError; end

    # General Errors
    class UnsafeValueError < Error; end

    # Intercom Errors
    class ServiceNotDefinedError < Error; end

    # Service Errors
    class NoPermissionsError < Error; end
    class MissingStoreError < Error; end

    # Channel Store Errors
    class ChannelStoreError < Error; end
    class InvalidStoreParamsError < ChannelStoreError; end
    class InvalidChannelError < ChannelStoreError; end

    # SDK Errors
    class RequestFailureError < Error; end

    # Channel Errors
    class ChannelNameMissingError < Error; end
    class ChannelTokenMissingError < Error; end
    class ChannelSecretMissingError < Error; end
  end # Errors
end # Ribbon::Intercom