require 'ostruct'
require 'base64'

module Ribbon::Intercom
  ##
  # Represents a collection of data to be passed between the service and client.
  class Packet < OpenStruct
    autoload(:MethodQueue, 'ribbon/intercom/packet/method_queue')

    class << self
      def decode(encoded_packet)
        hash = Marshal.load(Base64.strict_decode64(encoded_packet))
        raise Errors::InvalidEncodingError, hash.inspect unless hash.is_a?(Hash)
        new(hash)
      end
    end # Class Methods

    def initialize(params={})
      error = params.delete(:error)
      super(params)
      self.error = error if error
    end

    ##
    # Encode (marshal) the error before saving it. This allows the error to be
    # decoded on the client when requested, rather than decoded at the same time
    # the packet is decoded, which could cause problems if the error class doesn't
    # exist on the client.
    def error=(err)
      self._encoded_error = Marshal.dump(err)
    end

    ##
    # Decode the error, which may not exist on the client.
    # If the error class doesn't exist on the client, raise a ServerError.
    def error
      error? && Marshal.load(_encoded_error)
    rescue
      Errors::ServerError.new('unknown server error')
    end

    ##
    # Whether the packet contains an error
    def error?
      !!_encoded_error
    end

    def encode
      Base64.strict_encode64(Marshal.dump(to_h))
    end
  end # Packet
end # Ribbon::Intercom