module Ribbon::Intercom
  class Service
    module Channel::Stores
      class MockStore < Store
        def channels
          @__channels ||= {}
        end

        def lock
          @__lock ||= Mutex.new
        end

        def token_exists?(token)
          channels.key?(token)
        end

        def lookup_channel(token)
          channels[token]
        end

        def persist(channel)
          raise Errors::InvalidChannelError, channel.inspect unless channel.is_a?(Channel)
          channels[channel.token] = channel
        end

        def delete(channel)
          raise Errors::InvalidChannelError, channel.inspect unless channel.is_a?(Channel)
          channels.delete(channel.token)
          nil
        end

        def with_lock(channel, &block)
          # This is a global lock, not a per-channel lock, but this store is only
          # for testing so let's KISS.
          lock.synchronize(&block)
        end
      end # RedisStore
    end # Channel::Stores
  end # Service
end # Ribbon::Intercom