require 'date'

module Ribbon::Intercom
  module Utils
    autoload(:Signer, 'ribbon/intercom/utils/signer')
    autoload(:MethodChain, 'ribbon/intercom/utils/method_chain')
    autoload(:Mixins, 'ribbon/intercom/utils/mixins')

    BASIC_TYPES = [
      String, Symbol, TrueClass, FalseClass, Integer, Float, NilClass,
      Date, Time, DateTime,
      Hash, Array, Range
    ].freeze

    class << self
      def basic_type?(object)
        case object
        when *BASIC_TYPES then true
        else false
        end
      end

      def walk(object, context=nil, &block)
        case object
        when Hash
          Hash[
            object.map { |key, val|
              [walk(key, :hash_key, &block), walk(val, :hash_value, &block)]
            }
          ]
        when Array
          object.map { |obj| walk(obj, :array_elem, &block) }
        when Range
          Range.new(
            walk(object.begin, :range_begin, &block),
            walk(object.end, :range_end, &block),
            object.exclude_end?
          )
        else
          yield object, context
        end
      end

      ##
      # Raises an error if the object is or contains any non-basic types.
      def sanitize(object)
        walk(object) { |object|
          if basic_type?(object)
            object
          else
            raise Errors::UnsafeValueError, object.inspect
          end
        }
      end

      def symbolize_keys(hash)
        hash.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}
      end

      def classify(str)
        str.split("_").map(&:capitalize).join
      end

      ##
      # Returns an identifier for the method (e.g., A::B::C#method_name)
      def method_identifier(subject, method)
        scope = subject.is_a?(Class) ? subject.name : subject.class.name
        scope + (subject.is_a?(Class) ? '.' : '#') + method.to_s
      end
    end # Class methods
  end # Utils
end # Ribbon::Intercom