module Ribbon::Intercom
  module Utils
    class MethodChain
      class << self
        def begin(&block)
          new(&block)
        end
      end # Class Methods

      ##
      # Need to override all public instance methods so they can be captured correctly.
      public_instance_methods.each { |meth|
        define_method(meth) { |*args|
          _add_method(meth, *args)
        }
      }

      def initialize(&block)
        @_end_block = block
      end

      def end
        @_end_block.call(@_methods)
      end

      private

      def method_missing(meth, *args, &block)
        _add_method(meth, *args)
      end

      def _add_method(meth, *args)
        (@_methods ||= []) << [meth, *args]
        self
      end
    end # MethodChain
  end # Utils
end # Ribbon::Intercom